@@ -1,5 +1,6 @@
 import asyncio
 import functools
+import logging
 import sys
 import typing
 from types import TracebackType
@@ -16,6 +17,27 @@ if sys.version_info >= (3, 7):  # pragma: no cover
 else:  # pragma: no cover
     from aiocontextvars import ContextVar
 
+try:  # pragma: no cover
+    import click
+
+    # Extra log info for optional coloured terminal outputs.
+    LOG_EXTRA = {
+        "color_message": "Query: " + click.style("%s", bold=True) + " Args: %s"
+    }
+    CONNECT_EXTRA = {
+        "color_message": "Connected to database " + click.style("%s", bold=True)
+    }
+    DISCONNECT_EXTRA = {
+        "color_message": "Disconnected from database " + click.style("%s", bold=True)
+    }
+except ImportError:  # pragma: no cover
+    LOG_EXTRA = {}
+    CONNECT_EXTRA = {}
+    DISCONNECT_EXTRA = {}
+
+
+logger = logging.getLogger("databases")
+
 
 class Database:
     SUPPORTED_BACKENDS = {
@@ -25,16 +47,22 @@ class Database:
     }
 
     def __init__(
-        self, url: typing.Union[str, "DatabaseURL"], *, force_rollback: bool = False
+        self,
+        url: typing.Union[str, "DatabaseURL"],
+        *,
+        force_rollback: bool = False,
+        **options: typing.Any,
     ):
         self.url = DatabaseURL(url)
-        self._force_rollback = force_rollback
+        self.options = options
         self.is_connected = False
 
+        self._force_rollback = force_rollback
+
         backend_str = self.SUPPORTED_BACKENDS[self.url.dialect]
         backend_cls = import_from_string(backend_str)
         assert issubclass(backend_cls, DatabaseBackend)
-        self._backend = backend_cls(self.url)
+        self._backend = backend_cls(self.url, **self.options)
 
         # Connections are stored as task-local state.
         self._connection_context = ContextVar("connection_context")  # type: ContextVar
@@ -57,6 +85,9 @@ class Database:
         assert not self.is_connected, "Already connected."
 
         await self._backend.connect()
+        logger.info(
+            "Connected to database %s", self.url.obscure_password, extra=CONNECT_EXTRA
+        )
         self.is_connected = True
 
         if self._force_rollback:
@@ -74,6 +105,11 @@ class Database:
             await self._global_transaction.__aexit__()
 
         await self._backend.disconnect()
+        logger.info(
+            "Disconnected from database %s",
+            self.url.obscure_password,
+            extra=DISCONNECT_EXTRA,
+        )
         self.is_connected = False
 
     async def __aenter__(self) -> "Database":
@@ -154,6 +190,8 @@ class Connection:
         self._transaction_lock = asyncio.Lock()
         self._transaction_stack = []  # type: typing.List[Transaction]
 
+        self._query_lock = asyncio.Lock()
+
     async def __aenter__(self) -> "Connection":
         async with self._connection_lock:
             self._connection_counter += 1
@@ -176,12 +214,14 @@ class Connection:
     async def fetch_all(
         self, query: typing.Union[ClauseElement, str], values: dict = None
     ) -> typing.List[typing.Mapping]:
-        return await self._connection.fetch_all(self._build_query(query, values))
+        async with self._query_lock:
+            return await self._connection.fetch_all(self._build_query(query, values))
 
     async def fetch_one(
         self, query: typing.Union[ClauseElement, str], values: dict = None
     ) -> typing.Optional[typing.Mapping]:
-        return await self._connection.fetch_one(self._build_query(query, values))
+        async with self._query_lock:
+            return await self._connection.fetch_one(self._build_query(query, values))
 
     async def fetch_val(
         self,
@@ -189,25 +229,32 @@ class Connection:
         values: dict = None,
         column: typing.Any = 0,
     ) -> typing.Any:
-        row = await self._connection.fetch_one(self._build_query(query, values))
+        async with self._query_lock:
+            row = await self._connection.fetch_one(self._build_query(query, values))
         return None if row is None else row[column]
 
     async def execute(
         self, query: typing.Union[ClauseElement, str], values: dict = None
     ) -> typing.Any:
-        return await self._connection.execute(self._build_query(query, values))
+        async with self._query_lock:
+            return await self._connection.execute(self._build_query(query, values))
 
     async def execute_many(
         self, query: typing.Union[ClauseElement, str], values: list
     ) -> None:
         queries = [self._build_query(query, values_set) for values_set in values]
-        await self._connection.execute_many(queries)
+        async with self._query_lock:
+            await self._connection.execute_many(queries)
 
     async def iterate(
         self, query: typing.Union[ClauseElement, str], values: dict = None
     ) -> typing.AsyncGenerator[typing.Any, None]:
-        async for record in self._connection.iterate(self._build_query(query, values)):
-            yield record
+        async with self.transaction():
+            async with self._query_lock:
+                async for record in self._connection.iterate(
+                    self._build_query(query, values)
+                ):
+                    yield record
 
     def transaction(self, *, force_rollback: bool = False) -> "Transaction":
         return Transaction(self, force_rollback)
@@ -345,7 +392,10 @@ class DatabaseURL:
 
     @property
     def database(self) -> str:
-        return self.components.path.lstrip("/")
+        path = self.components.path
+        if path.startswith("/"):
+            path = path[1:]
+        return path
 
     @property
     def options(self) -> dict:
@@ -392,14 +442,17 @@ class DatabaseURL:
         components = self.components._replace(**kwargs)
         return self.__class__(components.geturl())
 
+    @property
+    def obscure_password(self) -> str:
+        if self.password:
+            return self.replace(password="********")._url
+        return self._url
+
     def __str__(self) -> str:
         return self._url
 
     def __repr__(self) -> str:
-        url = str(self)
-        if self.password:
-            url = str(self.replace(password="********"))
-        return f"{self.__class__.__name__}({repr(url)})"
+        return f"{self.__class__.__name__}({repr(self.obscure_password)})"
 
     def __eq__(self, other: typing.Any) -> bool:
         return str(self) == str(other)
